import argparse
import os

import numpy as np
from PIL import Image


class InconsistentImageError(Exception):
    """thrown when a selection of images are not the same dimensions"""
    pass


class InsufficientImagesError(Exception):
    """thrown when there are not enough images to average"""
    pass

def parse():
    parser = argparse.ArgumentParser()

    parser.add_argument("-f", "--folder", default=".", help="The folder to load images from")
    parser.add_argument("-o", "--output", default="out.png", help="filename of the resulting image")
    parser.add_argument("-v", "--verbose", action="store_true", help="print more verbose output")

    args = parser.parse_args()

    return vars(args)


def average(args):

    data = None
    files = [os.path.join(args["folder"], item) for item in os.listdir(args["folder"]) if any(ext in item for ext in ["png", "jpg", "bmp"])]
    processed_files = 0

    # break out if there are no images, to avoid division by zero below
    if not len(files):
        raise InsufficientImagesError("no images found in {}".format(os.path.abspath(args["folder"])))

    # The whole process is useless if there isnt at least two images, break out.
    if len(files) <= 1:
        raise InsufficientImagesError("not enough images to average")

    try:
        for path in files:
            im = Image.open(path)
            temp_arr = np.array(im).astype(np.uint64)

            if data is None: #populate the array
                data = temp_arr
            elif data.shape != temp_arr.shape: #throw if image dimensions differ
                raise InconsistentImageError("images must all be the same dimensions, and have the same bands E.G.(RGB, RGBA)")
            else:
                data += temp_arr
            processed_files += 1

            if args["verbose"]:
                print("loaded {}".format(path))

    except (TypeError, KeyboardInterrupt):
        # If the user attempts to cancel the process, we'll jump out of the loop
        # and save the result
        pass

    result = data / processed_files

    im = Image.fromarray(result.astype(np.uint8))

    if args["verbose"]:
        print("saving {}".format(args["output"]))

    im.save(args["output"])

    return result




import contextlib
import json
import os
import re
import subprocess

from setuptools import setup

VERSION_FILE = os.path.join(os.path.dirname(__file__), "version.json")


def _get_git_description():
    try:
        return subprocess.check_output(["git", "describe"]).decode("utf-8").strip()
    except subprocess.CalledProcessError:
        return None


def _create_version_from_description(git_description):
    match = re.match(r"(?P<tag>[\d\.]+)-(?P<offset>[\d]+)-(?P<sha>\w{8})", git_description)
    if not match:
        version = git_description
    else:
        version = "{tag}.post{offset}".format(**match.groupdict())
    return version


def get_version():
    with open(VERSION_FILE) as version_file:
        return json.loads(version_file.read())["version"]


@contextlib.contextmanager
def write_version():
    git_description = _get_git_description()

    version = _create_version_from_description(git_description) if git_description else None

    if version:
        with open(VERSION_FILE, "w") as version_file:
            version_file.write(json.dumps({"version": version}))
    yield


def main():
    with write_version():
        setup(
            name="imgavg",
            url="https://github.com/DoWhileGeek/imgavg",
            description="A command line utility that outputs the average of a number of pictures.",
            author="Joeseph Rodrigues",
            author_email="dowhilegeek@gmail.com",
            version=get_version(),
            install_requires=[
                "numpy==1.10.2",
                "Pillow==3.0.0"
            ],
            extras_require={
                "develop": [
                    "pytest==2.8.5",
                    "twine==1.5.0",
                ]},
            packages=[
                "imgavg"
            ],
            package_data={
                "imgavg": ["imgavg/*"],
            },
            scripts=["bin/imgavg"]
        )


if __name__ == "__main__":
    main()
